==> videosdk-1.0.0.tar.gz: lib/src/core/room/participant.dart <==
import 'package:events2/events2.dart';

import 'events.dart';
import 'stream.dart';

class Participant {
  late final EventEmitter _participantEventEmitter;
  late final EventEmitter _eventEmitter;
  int? _spatialLayer;
  int? _temporalLayer;

  String? _preferredQuality;

  late String id;
  late String displayName;
  late Map<String, Stream> streams;
  late bool isLocal;
  late Function _enablePeerMic,
      _disablePeerMic,
      _enablePeerCamera,
      _disablePeerCamera,
      _setConsumerQuality,
      _setViewPort,
      _removePeer;

  Participant({
    required this.id,
    required this.displayName,
    required this.isLocal,
    required EventEmitter eventEmitter,
    required Function enablePeerMic,
    required Function disablePeerMic,
    required Function enablePeerCamera,
    required Function disablePeerCamera,
    required Function setConsumerQuality,
    required Function setViewPort,
    required Function removePeer,
  }) {
    //
    _eventEmitter = eventEmitter;
    //
    _enablePeerMic = enablePeerMic;
    //
    _disablePeerMic = disablePeerMic;
    //
    _enablePeerCamera = enablePeerCamera;
    //
    _disablePeerCamera = disablePeerCamera;
    //
    _setConsumerQuality = setConsumerQuality;
    //
    _setViewPort = setViewPort;
    //
    _removePeer = removePeer;
    //
    _participantEventEmitter = EventEmitter();
    //
    streams = {};

    // handle stream-enabled
    _eventEmitter.on(
      "stream-enabled-$id",
      (data) {
        if (data['audio'] != null) {
          final consumer = data["audio"];

          final Stream newStream = Stream(
            consumer: consumer,
          );

          _addConsumer(newStream);
        } else if (data["renderer"] != null && data["video"] != null) {
          final consumer = data["video"];
          final renderer = data["renderer"];

          final Stream newStream = Stream(
            consumer: consumer,
            renderer: renderer,
          );

          _addConsumer(newStream);
        } else if (data["shareRenderer"] != null && data["share"] != null) {
          final consumer = data["share"];
          final shareRenderer = data["shareRenderer"];

          final Stream newStream = Stream(
            consumer: consumer,
            renderer: shareRenderer,
          );

          _addConsumer(newStream);
        }
      },
    );

    // handle stream-disabled
    _eventEmitter.on(
      "stream-disabled-$id",
      (data) {
        if (data['audioConsumerId'] != null) {
          final consumerId = data['audioConsumerId'];
          _removeConsumer(consumerId);
        } else if (data["renderer"] != null &&
            data["videoConsumerId"] != null) {
          final consumerId = data['videoConsumerId'];
          _removeConsumer(consumerId);
        } else if (data["shareRenderer"] != null &&
            data["shareConsumerId"] != null) {
          final consumerId = data['shareConsumerId'];
          _removeConsumer(consumerId);
        }
      },
    );

    // handle stream-paused
    _eventEmitter.on(
      "stream-paused-$id",
      (data) {
        String audioKeyType = 'audioConsumerId';
        String videoKeyType = 'videoConsumerId';
        String shareKeyType = 'shareConsumerId';

        if (data[audioKeyType] != null) {
          _pauseConsumer(audioKeyType, data);
        } else if (data[videoKeyType] != null) {
          _pauseConsumer(videoKeyType, data);
        } else if (data[shareKeyType] != null) {
          _pauseConsumer(shareKeyType, data);
        }
      },
    );

    // handle stream-resumed
    _eventEmitter.on(
      "stream-resumed-$id",
      (data) {
        String audioKeyType = 'audioConsumerId';
        String videoKeyType = 'videoConsumerId';
        String shareKeyType = 'shareConsumerId';

        if (data[audioKeyType] != null) {
          _resumeConsumer(audioKeyType, data);
        } else if (data[videoKeyType] != null) {
          _resumeConsumer(videoKeyType, data);
        } else if (data[shareKeyType] != null) {
          _resumeConsumer(shareKeyType, data);
        }
      },
    );

    // handle quality-changed
    _eventEmitter.on(
      "quality-changed-$id",
      (data) => _setLayers(data['spatialLayer'], data['temporalLayer']),
    );
  }

  String? get quality => _preferredQuality;

  on(Events event, handler) {
    _participantEventEmitter.on(event.parseToString(), handler);
  }

  _setLayers(int? spatialLayer, int? temporalLayer) {
    if (spatialLayer != null && temporalLayer != null) {
      _spatialLayer = spatialLayer;
      _temporalLayer = temporalLayer;
    }
  }

  _pauseConsumer(String idKeyType, data) {
    final consumer = data['consumer'];

    Stream? stream = streams[data[idKeyType]];

    if (stream != null) {
      Stream newStream = stream.copyWith(track: consumer);

      streams[newStream.id] = newStream;

      _participantEventEmitter.emit(
        "stream-paused",
        newStream,
      );
    }
  }

  _resumeConsumer(String idKeyType, data) {
    final consumer = data['consumer'];

    Stream? stream = streams[data[idKeyType]];

    if (stream != null) {
      Stream newStream = stream.copyWith(track: consumer);

      streams[newStream.id] = newStream;

      _participantEventEmitter.emit(
        "stream-resumed",
        newStream,
      );
    }
  }

  _addConsumer(Stream stream) {
    streams[stream.id] = stream;

    if (stream.kind == 'video' &&
        !isLocal &&
        _spatialLayer != null &&
        _temporalLayer != null) {
      _setConsumerQuality(stream.id, "s${_spatialLayer}t$_temporalLayer");
    }

    _participantEventEmitter.emit(
      "stream-enabled",
      stream,
    );
  }

  _removeConsumer(String consumerId) {
    final Stream? streamToRemove = streams[consumerId];

    if (streamToRemove != null) {
      streams.remove(consumerId);

      _participantEventEmitter.emit(
        "stream-disabled",
        streamToRemove,
      );
    }
  }

  unmuteMic() => _enablePeerMic(id);

  muteMic() => _disablePeerMic(id);

  enableCam() => _enablePeerCamera(id);

  disableCam() => _disablePeerCamera(id);

  setQuality(quality) {
    _preferredQuality = quality;
    streams.forEach((key, stream) {
      if (stream.kind == 'video' && !isLocal) {
        _setConsumerQuality(stream.id, quality);
      }
    });
  }

  setViewPort(width, height) {
    streams.forEach((key, stream) {
      if (stream.kind == 'video' && !isLocal) {
        _setViewPort(stream.id, width, height);
      }
    });
  }

  remove() => _removePeer(id);
}

==> videosdk-1.0.9.tar.gz: lib/src/core/room/participant.dart <==
import 'package:events2/events2.dart';

import 'events.dart';
import 'stream.dart';

class Participant {
  late final EventEmitter _participantEventEmitter;
  late final EventEmitter _eventEmitter;
  int? _spatialLayer;
  int? _temporalLayer;

  String? _preferredQuality;

  late String id;
  late String displayName;
  late Map<String, Stream> streams;
  late bool isLocal;
  late Function _enablePeerMic,
      _disablePeerMic,
      _enablePeerCamera,
      _disablePeerCamera,
      _setConsumerQuality,
      _setViewPort,
      _removePeer;

  Participant({
    required this.id,
    required this.displayName,
    required this.isLocal,
    required EventEmitter eventEmitter,
    required Function enablePeerMic,
    required Function disablePeerMic,
    required Function enablePeerCamera,
    required Function disablePeerCamera,
    required Function setConsumerQuality,
    required Function setViewPort,
    required Function removePeer,
  }) {
    //
    _eventEmitter = eventEmitter;
    //
    _enablePeerMic = enablePeerMic;
    //
    _disablePeerMic = disablePeerMic;
    //
    _enablePeerCamera = enablePeerCamera;
    //
    _disablePeerCamera = disablePeerCamera;
    //
    _setConsumerQuality = setConsumerQuality;
    //
    _setViewPort = setViewPort;
    //
    _removePeer = removePeer;
    //
    _participantEventEmitter = EventEmitter();
    //
    streams = {};

    // handle stream-enabled
    _eventEmitter.on(
      "stream-enabled-$id",
      (data) {
        if (data['audio'] != null) {
          final consumer = data["audio"];

          final Stream newStream = Stream(
            consumer: consumer,
          );

          _addConsumer(newStream);
        } else if (data["renderer"] != null && data["video"] != null) {
          final consumer = data["video"];
          final renderer = data["renderer"];

          final Stream newStream = Stream(
            consumer: consumer,
            renderer: renderer,
          );

          _addConsumer(newStream);
        } else if (data["shareRenderer"] != null && data["share"] != null) {
          final consumer = data["share"];
          final shareRenderer = data["shareRenderer"];

          final Stream newStream = Stream(
            consumer: consumer,
            renderer: shareRenderer,
          );

          _addConsumer(newStream);
        }
      },
    );

    // handle stream-disabled
    _eventEmitter.on(
      "stream-disabled-$id",
      (data) {
        if (data['audioConsumerId'] != null) {
          final consumerId = data['audioConsumerId'];
          _removeConsumer(consumerId);
        } else if (data["renderer"] != null &&
            data["videoConsumerId"] != null) {
          final consumerId = data['videoConsumerId'];
          _removeConsumer(consumerId);
        } else if (data["shareRenderer"] != null &&
            data["shareConsumerId"] != null) {
          final consumerId = data['shareConsumerId'];
          _removeConsumer(consumerId);
        }
      },
    );

    // handle stream-paused
    _eventEmitter.on(
      "stream-paused-$id",
      (data) {
        String audioKeyType = 'audioConsumerId';
        String videoKeyType = 'videoConsumerId';
        String shareKeyType = 'shareConsumerId';

        if (data[audioKeyType] != null) {
          _pauseConsumer(audioKeyType, data);
        } else if (data[videoKeyType] != null) {
          _pauseConsumer(videoKeyType, data);
        } else if (data[shareKeyType] != null) {
          _pauseConsumer(shareKeyType, data);
        }
      },
    );

    // handle stream-resumed
    _eventEmitter.on(
      "stream-resumed-$id",
      (data) {
        String audioKeyType = 'audioConsumerId';
        String videoKeyType = 'videoConsumerId';
        String shareKeyType = 'shareConsumerId';

        if (data[audioKeyType] != null) {
          _resumeConsumer(audioKeyType, data);
        } else if (data[videoKeyType] != null) {
          _resumeConsumer(videoKeyType, data);
        } else if (data[shareKeyType] != null) {
          _resumeConsumer(shareKeyType, data);
        }
      },
    );

    // handle quality-changed
    _eventEmitter.on(
      "quality-changed-$id",
      (data) {
        _setLayers(data['spatialLayer'], data['temporalLayer'],
            data['prevQuality'], data['currentQuality']);
      },
    );
  }

  String? get quality => _preferredQuality;

  on(Events event, handler) {
    _participantEventEmitter.on(event.parseToString(), handler);
  }

  _setLayers(int? spatialLayer, int? temporalLayer, String prevQuality,
      String currentQuality) {
    if (spatialLayer != null && temporalLayer != null) {
      _spatialLayer = spatialLayer;
      _temporalLayer = temporalLayer;
    }
    _participantEventEmitter.emit("video-quality-changed", <String, dynamic>{
      "prevQuality": prevQuality,
      "currentQuality": currentQuality,
    });
  }

  _pauseConsumer(String idKeyType, data) {
    final consumer = data['consumer'];

    Stream? stream = streams[data[idKeyType]];

    if (stream != null) {
      Stream newStream = stream.copyWith(track: consumer);

      streams[newStream.id] = newStream;

      _participantEventEmitter.emit(
        "stream-paused",
        newStream,
      );
    }
  }

  _resumeConsumer(String idKeyType, data) {
    final consumer = data['consumer'];

    Stream? stream = streams[data[idKeyType]];

    if (stream != null) {
      Stream newStream = stream.copyWith(track: consumer);

      streams[newStream.id] = newStream;

      _participantEventEmitter.emit(
        "stream-resumed",
        newStream,
      );
    }
  }

  _addConsumer(Stream stream) {
    streams[stream.id] = stream;

    if (stream.kind == 'video' &&
        !isLocal &&
        _spatialLayer != null &&
        _temporalLayer != null) {
      _setConsumerQuality(stream.id, "s${_spatialLayer}t$_temporalLayer");
    }

    _participantEventEmitter.emit(
      "stream-enabled",
      stream,
    );
  }

  _removeConsumer(String consumerId) {
    final Stream? streamToRemove = streams[consumerId];

    if (streamToRemove != null) {
      streams.remove(consumerId);

      _participantEventEmitter.emit(
        "stream-disabled",
        streamToRemove,
      );
    }
  }

  unmuteMic() => _enablePeerMic(id);

  muteMic() => _disablePeerMic(id);

  enableCam() => _enablePeerCamera(id);

  disableCam() => _disablePeerCamera(id);

  setQuality(quality) {
    _preferredQuality = quality;
    streams.forEach((key, stream) {
      if (stream.kind == 'video' && !isLocal) {
        _setConsumerQuality(stream.id, quality);
      }
    });
  }

  setViewPort(width, height) {
    streams.forEach((key, stream) {
      if (stream.kind == 'video' && !isLocal) {
        _setViewPort(stream.id, width, height);
      }
    });
  }

  remove() => _removePeer(id);
}
